@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2011,2015 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2015,2016 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -41,7 +41,7 @@
 #define CUR SP_TERMTYPE
 #endif
 
-MODULE_ID("$Id: lib_screen.c,v 1.72 2015/03/29 15:25:29 tom Exp $")
+MODULE_ID("$Id: lib_screen.c,v 1.80 2016/05/04 00:46:06 tom Exp $")
 
 #define MAX_SIZE 0x3fff		/* 16k is big enough for a window or pad */
 
@@ -56,7 +56,7 @@ MODULE_ID("$Id: lib_screen.c,v 1.72 2015/03/29 15:25:29 tom Exp $")
  * mistaken for the _cury/_curx pair of 16-bit numbers which start the old
  * format.  It happens to be unused in the file 5.22 database (2015/03/07).
  */
-static char my_magic[] =
+static const char my_magic[] =
 {'\210', '\210', '\210', '\210'};
 
 #if NCURSES_EXT_PUTWIN
@@ -73,19 +73,19 @@ typedef enum {
 } PARAM_TYPE;
 
 typedef struct {
-    const char *name;
+    const char name[11];
     attr_t attr;
 } SCR_ATTRS;
 
 typedef struct {
-    const char *name;
+    const char name[17];
     PARAM_TYPE type;
     size_t size;
     size_t offset;
 } SCR_PARAMS;
 
-#define DATA(name) { #name, A_##name }
-static SCR_ATTRS scr_attrs[] =
+#define DATA(name) { { #name }, A_##name }
+static const SCR_ATTRS scr_attrs[] =
 {
     DATA(NORMAL),
     DATA(STANDOUT),
@@ -111,9 +111,9 @@ static SCR_ATTRS scr_attrs[] =
 #undef DATA
 
 #define sizeof2(type,name) sizeof(((type *)0)->name)
-#define DATA(name, type) { #name, type, sizeof2(WINDOW, name), offsetof(WINDOW, name) }
+#define DATA(name, type) { { #name }, type, sizeof2(WINDOW, name), offsetof(WINDOW, name) }
 
-static SCR_PARAMS scr_params[] =
+static const SCR_PARAMS scr_params[] =
 {
     DATA(_cury, pSIZE),
     DATA(_curx, pSIZE),
@@ -179,6 +179,7 @@ read_txt(FILE *fp)
 		    result = 0;
 		    break;
 		}
+		result = buffer;
 	    }
 	    ch = fgetc(fp);
 	    if (ch == EOF)
@@ -220,16 +221,16 @@ decode_attr(char *source, attr_t *target, int *color)
 		++next;
 	    } else if (*next == 'C') {
 		int value = 0;
+		unsigned pair;
 		next++;
 		while (isdigit(UChar(*next))) {
 		    value = value * 10 + (*next++ - '0');
 		}
 		*target &= ~A_COLOR;
-		if (value > 256) {
-		    *target |= COLOR_PAIR(255);
-		} else {
-		    *target |= COLOR_PAIR(value);
-		}
+		pair = (unsigned) ((value > 256)
+				   ? COLOR_PAIR(255)
+				   : COLOR_PAIR(value));
+		*target |= pair;
 		*color = value;
 	    } else {
 		while (isalnum(UChar(*next))) {
@@ -315,7 +316,7 @@ static char *
 decode_chtype(char *source, chtype fillin, chtype *target)
 {
     attr_t attr = ChAttrOf(fillin);
-    int color = PAIR_NUMBER(attr);
+    int color = PAIR_NUMBER((int) attr);
     int value;
 
     T(("decode_chtype '%s'", source));
@@ -351,7 +352,7 @@ decode_cchar(char *source, cchar_t *fillin, cchar_t *target)
     while (source[0] == MARKER && source[1] == APPEND) {
 	source += 2;
 	source = decode_char(source, &value);
-	if (append++ < CCHARW_MAX) {
+	if (++append < CCHARW_MAX) {
 	    chars[append] = (wchar_t) value;
 	}
     }
@@ -587,7 +588,7 @@ NCURSES_SP_NAME(getwin) (NCURSES_SP_DCLx FILE *filep)
 	}
 #if NCURSES_EXT_PUTWIN
 	else {
-	    char *txt;
+	    char *txt = 0;
 	    bool success = TRUE;
 	    NCURSES_CH_T prior = blank;
 
@@ -669,7 +670,7 @@ encode_attr(char *target, attr_t source, attr_t prior)
 	if ((source & A_COLOR) != (prior & A_COLOR)) {
 	    if (!first)
 		*target++ = '|';
-	    sprintf(target, "C%d", PAIR_NUMBER(source));
+	    sprintf(target, "C%d", PAIR_NUMBER((int) source));
 	    target += strlen(target);
 	}
 
@@ -695,7 +696,7 @@ encode_cell(char *target, CARG_CH_T source, CARG_CH_T previous)
     }
 #endif
     for (n = 0; n < SIZEOF(source->chars); ++n) {
-	unsigned uch = source->chars[n];
+	unsigned uch = (unsigned) source->chars[n];
 	if (uch == 0)
 	    continue;
 	if (n) {
@@ -735,7 +736,7 @@ encode_cell(char *target, CARG_CH_T source, CARG_CH_T previous)
     target += strlen(target);
     *target++ = MARKER;
     if (ch < 32 || ch >= 127) {
-	sprintf(target, "%03o", ch);
+	sprintf(target, "%03o", UChar(ch));
     } else {
 	switch (ch) {
 	case ' ':
@@ -746,11 +747,10 @@ encode_cell(char *target, CARG_CH_T source, CARG_CH_T previous)
 	    *target = '\0';
 	    break;
 	default:
-	    sprintf(--target, "%c", ch);
+	    sprintf(--target, "%c", UChar(ch));
 	    break;
 	}
     }
-    target += strlen(target);
 #endif
 }
 #endif
@@ -843,15 +843,22 @@ putwin(WINDOW *win, FILE *filep)
 		|| ferror(filep))
 		returnCode(code);
 	    for (x = 0; x <= win->_maxx; x++) {
+#if NCURSES_WIDECHAR
 		int len = wcwidth(data[x].chars[0]);
 		encode_cell(buffer, CHREF(data[x]), CHREF(last_cell));
 		last_cell = data[x];
 		PUTS(buffer);
 		if (len > 1)
 		    x += (len - 1);
+#else
+		encode_cell(buffer, CHREF(data[x]), CHREF(last_cell));
+		last_cell = data[x];
+		PUTS(buffer);
+#endif
 	    }
 	    PUTS("\n");
 	}
+	code = OK;
     }
 #else
     /*
